@@ -37,6 +37,11 @@ struct cancel_info {
   uv_timer_t timer_handle;
 };
 
+struct random_info {
+  uv_random_t random_req;
+  char buf[1];
+};
+
 static unsigned fs_cb_called;
 static unsigned done_cb_called;
 static unsigned done2_cb_called;
@@ -60,7 +65,10 @@ static void saturate_threadpool(void) {
   char buf[64];
   size_t i;
 
-  snprintf(buf, sizeof(buf), "UV_THREADPOOL_SIZE=%zu", ARRAY_SIZE(pause_reqs));
+  snprintf(buf,
+           sizeof(buf),
+           "UV_THREADPOOL_SIZE=%lu",
+           (unsigned long)ARRAY_SIZE(pause_reqs));
   putenv(buf);
 
   loop = uv_default_loop();
@@ -79,8 +87,39 @@ static void unblock_threadpool(void) {
 }
 
 
+static int known_broken(uv_req_t* req) {
+  if (req->type != UV_FS)
+    return 0;
+
+#ifdef __linux__
+  /* TODO(bnoordhuis) make cancellation work with io_uring */
+  switch (((uv_fs_t*) req)->fs_type) {
+    case UV_FS_CLOSE:
+    case UV_FS_FDATASYNC:
+    case UV_FS_FSTAT:
+    case UV_FS_FSYNC:
+    case UV_FS_LINK:
+    case UV_FS_LSTAT:
+    case UV_FS_MKDIR:
+    case UV_FS_OPEN:
+    case UV_FS_READ:
+    case UV_FS_RENAME:
+    case UV_FS_STAT:
+    case UV_FS_SYMLINK:
+    case UV_FS_WRITE:
+    case UV_FS_UNLINK:
+      return 1;
+    default:  /* Squelch -Wswitch warnings. */
+      break;
+  }
+#endif
+
+  return 0;
+}
+
+
 static void fs_cb(uv_fs_t* req) {
-  ASSERT(req->result == UV_ECANCELED);
+  ASSERT(known_broken((uv_req_t*) req) || req->result == UV_ECANCELED);
   uv_fs_req_cleanup(req);
   fs_cb_called++;
 }
@@ -90,7 +129,7 @@ static void getaddrinfo_cb(uv_getaddrinfo_t* req,
                            int status,
                            struct addrinfo* res) {
   ASSERT(status == UV_EAI_CANCELED);
-  ASSERT(res == NULL);
+  ASSERT_NULL(res);
   uv_freeaddrinfo(res);  /* Should not crash. */
 }
 
@@ -100,8 +139,8 @@ static void getnameinfo_cb(uv_getnameinfo_t* handle,
                            const char* hostname,
                            const char* service) {
   ASSERT(status == UV_EAI_CANCELED);
-  ASSERT(hostname == NULL);
-  ASSERT(service == NULL);
+  ASSERT_NULL(hostname);
+  ASSERT_NULL(service);
 }
 
 
@@ -125,7 +164,7 @@ static void timer_cb(uv_timer_t* handle) {
 
   for (i = 0; i < ci->nreqs; i++) {
     req = (uv_req_t*) ((char*) ci->reqs + i * ci->stride);
-    ASSERT(0 == uv_cancel(req));
+    ASSERT(known_broken(req) || 0 == uv_cancel(req));
   }
 
   uv_close((uv_handle_t*) &ci->timer_handle, NULL);
@@ -140,6 +179,19 @@ static void nop_done_cb(uv_work_t* req, int status) {
 }
 
 
+static void nop_random_cb(uv_random_t* req, int status, void* buf, size_t len) {
+  struct random_info* ri;
+
+  ri = container_of(req, struct random_info, random_req);
+
+  ASSERT(status == UV_ECANCELED);
+  ASSERT(buf == (void*) ri->buf);
+  ASSERT(len == sizeof(ri->buf));
+
+  done_cb_called++;
+}
+
+
 TEST_IMPL(threadpool_cancel_getaddrinfo) {
   uv_getaddrinfo_t reqs[4];
   struct cancel_info ci;
@@ -168,7 +220,7 @@ TEST_IMPL(threadpool_cancel_getaddrinfo) {
   ASSERT(0 == uv_run(loop, UV_RUN_DEFAULT));
   ASSERT(1 == timer_cb_called);
 
-  MAKE_VALGRIND_HAPPY();
+  MAKE_VALGRIND_HAPPY(loop);
   return 0;
 }
 
@@ -204,7 +256,30 @@ TEST_IMPL(threadpool_cancel_getnameinfo) {
   ASSERT(0 == uv_run(loop, UV_RUN_DEFAULT));
   ASSERT(1 == timer_cb_called);
 
-  MAKE_VALGRIND_HAPPY();
+  MAKE_VALGRIND_HAPPY(loop);
+  return 0;
+}
+
+
+TEST_IMPL(threadpool_cancel_random) {
+  struct random_info req;
+  uv_loop_t* loop;
+
+  saturate_threadpool();
+  loop = uv_default_loop();
+  ASSERT(0 == uv_random(loop,
+                        &req.random_req,
+                        &req.buf,
+                        sizeof(req.buf),
+                        0,
+                        nop_random_cb));
+  ASSERT(0 == uv_cancel((uv_req_t*) &req));
+  ASSERT(0 == done_cb_called);
+  unblock_threadpool();
+  ASSERT(0 == uv_run(loop, UV_RUN_DEFAULT));
+  ASSERT(1 == done_cb_called);
+
+  MAKE_VALGRIND_HAPPY(loop);
   return 0;
 }
 
@@ -228,7 +303,7 @@ TEST_IMPL(threadpool_cancel_work) {
   ASSERT(1 == timer_cb_called);
   ASSERT(ARRAY_SIZE(reqs) == done2_cb_called);
 
-  MAKE_VALGRIND_HAPPY();
+  MAKE_VALGRIND_HAPPY(loop);
   return 0;
 }
 
@@ -261,7 +336,7 @@ TEST_IMPL(threadpool_cancel_fs) {
   ASSERT(0 == uv_fs_lstat(loop, reqs + n++, "/", fs_cb));
   ASSERT(0 == uv_fs_mkdir(loop, reqs + n++, "/", 0, fs_cb));
   ASSERT(0 == uv_fs_open(loop, reqs + n++, "/", 0, 0, fs_cb));
-  ASSERT(0 == uv_fs_read(loop, reqs + n++, 0, &iov, 1, 0, fs_cb));
+  ASSERT(0 == uv_fs_read(loop, reqs + n++, -1, &iov, 1, 0, fs_cb));
   ASSERT(0 == uv_fs_scandir(loop, reqs + n++, "/", 0, fs_cb));
   ASSERT(0 == uv_fs_readlink(loop, reqs + n++, "/", fs_cb));
   ASSERT(0 == uv_fs_realpath(loop, reqs + n++, "/", fs_cb));
@@ -272,7 +347,7 @@ TEST_IMPL(threadpool_cancel_fs) {
   ASSERT(0 == uv_fs_symlink(loop, reqs + n++, "/", "/", 0, fs_cb));
   ASSERT(0 == uv_fs_unlink(loop, reqs + n++, "/", fs_cb));
   ASSERT(0 == uv_fs_utime(loop, reqs + n++, "/", 0, 0, fs_cb));
-  ASSERT(0 == uv_fs_write(loop, reqs + n++, 0, &iov, 1, 0, fs_cb));
+  ASSERT(0 == uv_fs_write(loop, reqs + n++, -1, &iov, 1, 0, fs_cb));
   ASSERT(n == ARRAY_SIZE(reqs));
 
   ASSERT(0 == uv_timer_init(loop, &ci.timer_handle));
@@ -282,7 +357,7 @@ TEST_IMPL(threadpool_cancel_fs) {
   ASSERT(1 == timer_cb_called);
 
 
-  MAKE_VALGRIND_HAPPY();
+  MAKE_VALGRIND_HAPPY(loop);
   return 0;
 }
 
@@ -300,6 +375,39 @@ TEST_IMPL(threadpool_cancel_single) {
   ASSERT(0 == uv_run(loop, UV_RUN_DEFAULT));
   ASSERT(1 == done_cb_called);
 
-  MAKE_VALGRIND_HAPPY();
+  MAKE_VALGRIND_HAPPY(loop);
+  return 0;
+}
+
+
+static void after_busy_cb(uv_work_t* req, int status) {
+  ASSERT_OK(status);
+  done_cb_called++;
+}
+
+static void busy_cb(uv_work_t* req) {
+  uv_sem_post((uv_sem_t*) req->data);
+  /* Assume that calling uv_cancel() takes less than 10ms. */
+  uv_sleep(10);
+}
+
+TEST_IMPL(threadpool_cancel_when_busy) {
+  uv_sem_t sem_lock;
+  uv_work_t req;
+
+  req.data = &sem_lock;
+
+  ASSERT_OK(uv_sem_init(&sem_lock, 0));
+  ASSERT_OK(uv_queue_work(uv_default_loop(), &req, busy_cb, after_busy_cb));
+
+  uv_sem_wait(&sem_lock);
+
+  ASSERT_EQ(uv_cancel((uv_req_t*) &req), UV_EBUSY);
+  ASSERT_OK(uv_run(uv_default_loop(), UV_RUN_DEFAULT));
+  ASSERT_EQ(done_cb_called, 1);
+
+  uv_sem_destroy(&sem_lock);
+
+  MAKE_VALGRIND_HAPPY(uv_default_loop());
   return 0;
 }
